/** @file modbuscli.c MODBUS/UDP client sample for the MSXE3511
 *
 * FC16 queries are for setting things, FC3 for fetching values
 *
 * This code doesn't handle timeouts.
 */
#ifdef WIN32
#include "../../Interface_Library/msxe351x_modbus_clientlib.h"
#else
#include <msxe351x_modbus_clientlib.h>
#endif

#include <modbusclient.h>
#include <commonclient.h>

#include <addidata_console.h>
#include <addidata_time.h>
#include <addidata_network.h>

//----------------------------------------------------------------------------------------------------------------------------------
int sample_MX351x__AnalogOutputGetStatus(struct modbus * modbus, uint32_t option, uint32_t *status, uint32_t *info)
{
	struct MX351x__AnalogOutputGetStatus_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX351x__AnalogOutputGetStatus(modbus, &Response);
	*status = Response.Status;
	*info = Response.Info;
	return handle_result_of_FC3_query(modbus, ret, "modbus_call_MX351x__AnalogOutputGetStatusEx");
}

//----------------------------------------------------------------------------------------------------------------------------------
int sample_modbus_call_MX351x__getNumberOfChannelsEx(struct modbus * modbus)
{
	struct MX351x__getNumberOfChannelsEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX351x__getNumberOfChannelsEx(modbus,&Response);
	if (!handle_result_of_FC3_query(modbus, ret, "modbus_call_MX351x__getNumberOfChannelsEx"))
	{
		printf("number of channels on the module: %d\n",Response.ChannelNumber);
		return 0;
	}
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MSXE351x__GetAnalogOutputConfigEx(struct modbus * modbus,
					   uint16_t channel,
					   uint8_t* OutputType,
					   uint8_t* Polarity,
					   uint16_t* Value)
{
	int ret = 0;
	struct MX351x__GetAnalogOutputConfig0Ex_parameters_t Response;
	memset(&Response, 0, sizeof(Response));

	switch(channel)
	{
		if (channel == 0)
			ret = modbus_call_MX351x__GetAnalogOutputConfig0Ex(modbus,&Response);
		else if (channel == 1)
			ret = modbus_call_MX351x__GetAnalogOutputConfig1Ex(modbus,(struct MX351x__GetAnalogOutputConfig1Ex_parameters_t *)&Response);
		else if (channel == 2)
			ret = modbus_call_MX351x__GetAnalogOutputConfig2Ex(modbus,(struct MX351x__GetAnalogOutputConfig2Ex_parameters_t *)&Response);
		else if (channel == 3)
			ret = modbus_call_MX351x__GetAnalogOutputConfig3Ex(modbus,(struct MX351x__GetAnalogOutputConfig3Ex_parameters_t *)&Response);
		else if (channel == 4)
			ret = modbus_call_MX351x__GetAnalogOutputConfig4Ex(modbus,(struct MX351x__GetAnalogOutputConfig4Ex_parameters_t *)&Response);
		else if (channel == 5)
			ret = modbus_call_MX351x__GetAnalogOutputConfig5Ex(modbus,(struct MX351x__GetAnalogOutputConfig5Ex_parameters_t *)&Response);
		else if (channel == 6)
			ret = modbus_call_MX351x__GetAnalogOutputConfig6Ex(modbus,(struct MX351x__GetAnalogOutputConfig6Ex_parameters_t *)&Response);
		else if (channel == 7)
			ret = modbus_call_MX351x__GetAnalogOutputConfig7Ex(modbus,(struct MX351x__GetAnalogOutputConfig7Ex_parameters_t *)&Response);
		else
			printf("Channel selected is wrong %s %d", __FUNCTION__, __LINE__);
	}
	*OutputType = Response.OutputType;
	*Polarity = Response.Polarity;
	*Value = Response.Value;

	return handle_result_of_FC3_query(modbus, ret, "MX351x__GetAnalogOutputConfig_parameters_t");
}

int Write1_Worked(struct modbus* modbus,
		  uint16_t Channel,
		  uint8_t OutputType,
		  uint8_t Polarity,
		  uint16_t Value,
		  uint16_t trigger)
{
	uint8_t real_OutputType;
	uint8_t real_Polarity;
	uint16_t real_Value;

	printf("Checking back configuration: ");

	if (sample_MSXE351x__GetAnalogOutputConfigEx(modbus, Channel, &real_OutputType, &real_Polarity, &real_Value) != 0)
	{
		printf("%s %d sample_MSXE351x__GetAnalogOutputConfigEx error\n", __FUNCTION__, __LINE__);
		return 1;
	}

	if (real_OutputType!=OutputType)
	{
		printf("! channel %d: OutputType read from module (%d) differs of expected (%d)\n", Channel, real_OutputType,OutputType);
		return 1;
	}
	if (real_Polarity!=Polarity)
	{
		printf("! channel %d: Polarity read from module (%d) differs of expected (%d)\n", Channel, real_Polarity,Polarity);
		return 1;
	}
	if (real_Value!=Value)
	{
		printf("! channel %d: Value read from module (%d) differs of expected (%d)\n", Channel, real_Value,Value);
		return 1;
	}

	printf("\nchannel %d OK\n",Channel);
	return 0;
}

int wait_trigger(struct modbus* modbus)
{

	key_thread_handle_t kt;
	unsigned int stop = 0;
	uint32_t status = 0;
	uint32_t info = 0;
	wait_key_input_thread_parameters_t ktparams;
	ktparams.character = 27; /* ESC */
	ktparams.flag = &stop;
	kt = addidata_start_key_thread(&ktparams);

	while(sample_MX351x__AnalogOutputGetStatus(modbus, 0, &status, &info) == 0)
	{
		if (stop)
			break;

		if (status == 0)
		{
			printf("\nready to receive a new value\n");
			break;
		}
		else if (status == 1)
		{
			printf("\rWaiting for a trigger");
		}
		else
		{
			printf("\nError output not available\n");
			return -1;
		}
	}
	addidata_stop_key_thread(kt);
	return 0;
}

//---------------------------------------------------------------------------------
int sample_MSXE351x__AnalogOutputWriteMoreValuesEx(struct modbus * modbus,
						   uint8_t channelMask,
						   uint8_t triggerMask,
						   uint8_t triggerEdgeSelection,
						   uint16_t triggerCount,
						   uint8_t* outputType,
						   uint8_t* polarity,
						   uint16_t *values)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	uint16_t channelConfiguration = (uint16_t)channelMask;
	int ret = modbus_call_MSXE351x__AnalogOutputWriteMoreValuesEx(modbus,
								      channelConfiguration,
								      triggerMask,
								      triggerEdgeSelection,
								      triggerCount,
								      outputType,
								      polarity,
								      values,
								      &CommandStatus);

	if ((ret == 0) && (triggerMask != 0))
		wait_trigger(modbus); 

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE351x__AnalogOutputWriteMoreValuesEx");
}

void usageAnalogOutputs(char *name, uint32_t sampleSelection)
{
	printf("Write %s analog output usage:\n\n", (sampleSelection != 0) ? "more" : "1");
	printf("%s ADDRESS SAMPLE %s OUTPUTTYPE POLARITY TRIGGERMASK TRIGGEREDGE TRIGGERCOUNT VALUE\n\n", name, (sampleSelection != 0) ? "CHANNELMASK" : "CHANNEL");
	if (sampleSelection != 0)
		printf("where CHANNELMASK is value of channel(s) to set (0 to (2^(Module number of channel) - 1))\n\n");
	else
		printf("where CHANNEL is value of channel(s) to set (0 to system number of channels)\n\n");

	printf("where OUTPUTTYPE is \n\t0: Voltage\n\t1: Current\n\t2: System default\n\n");
	printf("where POLARITY is \n\t0: Unipolar\n\t1: Bipolar\n\n");
	printf("where TRIGGERMASK is \n\t0: Output trigger not used\n\t1: Hardware trigger input\n\t2: Synchro input\n\n");
	printf("where TRIGGEREDGE is \n\t1: Rising edge\n\t2: Falling edge\n\t3: Both\n\n");
	printf("where TRIGGERCOUNT is the number of trigger events before the action occurs\n\t1: All trigger events start the action\n\t2->65535: Possible values\n\n");
	printf("where VALUE is the value of the channel(s) to set:\n\tBipolar: 0 to 0xFFFF\n\tUnipolar: 0 to 0x7FFF\n\n");
}

void help(const char* name)
{
	printf("%s ADDRESS SAMPLE\n", name);
	printf("where SAMPLE is\n");
	printf("\t0 Write 1 analog value\n");
	printf("\t1 Write more analog values\n");
	printf("Enter %s ADDRESS SAMPLE for more information about sample usage\n", name);
}

//---------------------------------------------------------------------------------
int sample_MSXE351x__AnalogOutputWrite1ValueEx(struct modbus * modbus,
					       uint8_t  channel,
					       uint8_t  outputType,
					       uint8_t  polarity,
					       uint8_t  triggerMask,
					       uint8_t  triggerEdgeSelection,
					       uint16_t triggerCount,
					       uint16_t value)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = modbus_call_MSXE351x__AnalogOutputWrite1ValueEx(modbus,
								  channel,
								  outputType,
								  polarity,
								  triggerMask,
								  triggerEdgeSelection,
								  triggerCount,
								  value,
								  &CommandStatus);

	return (handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MSXE351x__AnalogOutputWrite1ValueEx") ? ret : Write1_Worked(modbus, channel, outputType, polarity, value, triggerMask));
}

int sampleAnalogOutputs(int argc, char** argv, uint32_t sampleSelection)
{
	struct modbus modbus;
	uint8_t channel              = 0;
	uint8_t outputType           = 0;
	uint8_t polarity             = 0;
	uint8_t triggerMask          = 0;
	uint8_t triggerEdgeSelection = 0;
	uint16_t triggerCount         = 0;
	uint16_t value                = 0;

	if (argc != 10)
	{
		printf("wrong number of %s parameters\n", argv[0]);
		usageAnalogOutputs(argv[0], sampleSelection);
		exit(1);
	}

	channel = (uint8_t)atoi(argv[3]);
	if ((sampleSelection == 0) && (channel > 7))
	{
		printf("Wrong channel selection must be < 7\n");
		exit(1);
	}

	outputType = (uint8_t)atoi(argv[4]);
	if (outputType > 2)
	{
		printf("Wrong output type selection\n");
		exit(1);
	}

	polarity = (uint8_t)atoi(argv[5]);
	if (polarity > 1)
	{
		printf("Wrong polarity selection\n");
		exit(1);
	}

	triggerMask = (uint8_t)atoi(argv[6]);
	if (triggerMask > 3)
	{
		printf("Wrong trigger mask selection\n");
		exit(1);
	}

	triggerEdgeSelection = (uint8_t)atoi(argv[7]);
	if (triggerEdgeSelection > 3)
	{
		printf("Wrong trigger edge selection\n");
		exit(1);
	}

	if (atoi(argv[8]) > 0xFFFF)
	{
		printf("Wrong triggerCount selection");
		exit(1);
	}
	triggerCount = (uint16_t)atoi(argv[8]);

	if (atoi(argv[9]) > 0xFFFF)
	{
		printf("Wrong value selection\n");
		exit(1);
	}

	value = (uint16_t)atoi(argv[9]);
	if ((polarity == 0) && (value > 0x7FFF))
	{
		printf("Wrong channel value selection\n");
		exit(1);
	}

	modbus_init(&modbus);
	switch (modbus_connect(&modbus, argv[1], SOCK_DGRAM, 0))
	{
		case 0: /* success */
			break;
		case -1: /* system error */
			perror("modbus_connect");
			exit(1);
		case -2: /* parameter is incorrect */
			printf("modbus_connect: incorrect parameter\n");
			exit(1);
		case -3: /* incorrect address */
			printf("incorrect address\n");
			exit(1);
	}

	if (sampleSelection == 0)
	{
		if (sample_MSXE351x__AnalogOutputWrite1ValueEx(&modbus,
							       channel,
							       outputType,
							       polarity,
							       triggerMask,
							       triggerEdgeSelection,
							       triggerCount,
							       value) != 0)
		{
			printf("%s %d sample_MSXE351x__AnalogOutputWrite1ValueEx error\n", __FUNCTION__, __LINE__);
			exit(1);
		}
	}
	else
	{
		uint16_t values[8];
		uint8_t outputTypes[8];
		uint8_t polarities[8];

		int i = 0;
		for (i = 0; i < (sizeof(values) / sizeof(uint16_t)); i++)
		{
			values[i] = value;
			outputTypes[i] = outputType;
			polarities[i] = polarity;
		}
		if (sample_MSXE351x__AnalogOutputWriteMoreValuesEx(&modbus,
								   channel,
								   triggerMask,
								   triggerEdgeSelection,
								   triggerCount,
								   outputTypes,
								   polarities,
								   values) != 0)
		{
			printf("%s %d sample_MSXE351x__AnalogOutputWriteMoreValuesEx error\n", __FUNCTION__, __LINE__);
			exit(1);
		}
	}

	return 0;
}

int main(int argc, char** argv)
{
	uint32_t sampleSelection = 0;

	if (argc < 3)
	{
		help(argv[0]);
		exit(1);
	}

	sampleSelection = (uint32_t)atoi(argv[2]);

	// Sample selection
	if (sampleSelection < 2)
		sampleAnalogOutputs(argc, argv, sampleSelection);

	return 0;
}
