#include <unistd.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <errno.h>
#include <MSXE351x.nsmap> // this file must be included only once in the project
#include <assert.h>
#include <sys/stat.h>
#include <signal.h>

#include "utils.h"

/* IP Address of the module */
char * ipAddress = "192.168.99.99";

/* please see utils.h */
int DUMMY = 0;

//------------------------------------------------------------------------------------------
/** writeOneFunction : write value on only one channel to the analog output of the MSX-E351x
 *
 * @param[in] soapEnv The current SOAP environment
 * @param[in] endPoint : SOAP endpoint
 * @param[in] nbrOfChannel : Number of channel on the board
 *
 * @return 0
 *
 */
int writeOneFunction(struct soap *soapEnv, char *endPoint, unsigned long nbrOfChannel)
{
    /* Set the port configuration */
    struct MSXE351x__Response response;
    struct MSXE351x__AnalogOutputGetStatusResponse statusResponse;
    unsigned long channel = 0;
    unsigned long outputType = 0;
    unsigned long polarity = 0;
    unsigned long triggerMask = 0;
    unsigned long triggerEdgeSelection = 0;
    unsigned long triggerCount = 0;
    unsigned long channelValue = 0x7FFF;
    unsigned long continueOk = 0;

    response.iReturnValue = 0;
    statusResponse.sResponse.iReturnValue = 0;

    printf("\n");
    do
    {
        printf("\n Do You want to write another value ? (0: No, 1 : Yes) :");
        scanf("%lu", &continueOk);
    } while (continueOk > 1);

    if (continueOk)
    {
        do
        {
            printf("\nEnter the channel do You want to write to (0 to %lu) :", nbrOfChannel - 1);
            scanf("%lu", &channel);
        } while (channel >= nbrOfChannel);
        do
        {
            printf("\nEnter the channel configuration do You want to use (0:Voltage, 1:Current) :");
            scanf("%lu", &outputType);
        } while (outputType > 1);
        do
        {
            printf("\nEnter the channel polarity do You want to use (0:Unipolar, 1:Bipolar) :");
            scanf("%lu", &polarity);
        } while (polarity > 1);
        do
        {
            printf("\nDo You want to use an extern trigger ? (0:no trigger use, 1:Digital Input Trigger, 2:Synchro Trigger) :");
            scanf("%lu", &triggerMask);
        } while (triggerMask > 2);

        if (triggerMask == 1)
        {
            do
            {
                printf("\nSelect the edge to react for the digital input trigger (1:rising edge, 2:falling edge, 3:both edge) :");
                scanf("%lu", &triggerEdgeSelection);
            } while ((triggerEdgeSelection == 0) || (triggerEdgeSelection > 3));
            do
            {
                printf("\nSelect the digital input trigger count value (min 1):");
                scanf("%lu", &triggerCount);
            } while (triggerCount == 0);

        }

        do
        {
            if (polarity == 0)
                printf("\nEnter the output value to write in Hex (0 to 0x7FFF) : 0x");
            else
                printf("\nEnter the output value to write in Hex (0 to 0xFFFF) : 0x");
            scanf("%lX", &channelValue);
        } while (((channelValue > 0x7FFF) && (polarity == 0)) || ((channelValue > 0xFFFF) && (polarity == 1)));

        /* Wrtie a value to the channel output */
        assert_gsoap(soapEnv, soap_call_MSXE351x__AnalogOutputWrite1Value(soapEnv, endPoint, NULL, channel, outputType, polarity, triggerMask, triggerEdgeSelection, triggerCount, channelValue, 0, 0, 0, &response));
        assert_soap_command(soapEnv, (struct DefaultResponse*) &response);

        /* check if a trigger is activated */
        if (triggerMask == 0)
            printf("Write %lu on channel %lu\n", channelValue, channel);
        else
        {
            printf("\n The output value wait for a trigger ! Press ESC to abort...");

            for (;;)
            {
                assert_gsoap(soapEnv, soap_call_MSXE351x__AnalogOutputWrite1Value(soapEnv, endPoint, NULL, channel, outputType, polarity, triggerMask, triggerEdgeSelection, triggerCount, channelValue, 0, 0, 0, &response));
                assert_soap_command(soapEnv, (struct DefaultResponse*) &response);
                if (statusResponse.sResponse.iReturnValue == 0)
                {
                    if (statusResponse.ulStatus == 0)
                    {
                        printf("\n The trigger occur, the output is ready to receive another value");
                        return 0;
                    }
                    else
                    {
                        if (kbhit())
                        {
                            if (getch() == 27)
                                return 1;
                        }
                    }

                }

            }

        }
    }
    else
    {
        return 1;
    }

    return 0;
}

//-----------------------------------------------------------------------------------------
/** signal handler : stops the current acquisition and cleanup the SOAP connection.
 *
 */
static void do_signal(int sig)
{
    struct soap soapEnv;
    mx_soap_init(&soapEnv);

    printf("signal received:%s\n", strsignal(sig));

    /* note: the soapenv variable should have been correctly initialised before the signal handler was set */
    mx_soap_clean(&soapEnv);

    exit(1);
}

//-----------------------------------------------------------------------------------------
/** For all signals set the handler do_signal.
 *
 * This allows to handle CTRL+C nicely
 *
 * @note This function must be called after the first call to a function of the soap_call() familly.
 */
static void setup_signal(void)
{
    struct sigaction action;
    /* catch all signal */
    sigfillset(&action.sa_mask); /* all other signal will be blocked when handling the one received */
    action.sa_handler = do_signal;
    action.sa_flags = 0;
    {
        int i;
        for (i = 1; i < NSIG; i++)
        {
            if (SIGKILL == i) /* this signal can not be blocked */
                continue;
            if (SIGSTOP == i) /* this signal can not be blocked */
                continue;
            sigaction(i, &action, NULL); /* TODO: manage error */
        }
    }
}

int main(int argc, char ** argv)
{
    char endPoint[30];
    struct soap soapEnv;
    struct MXCommon__ByteArrayResponse response;
    int nbrOfChannel = 8;
    char * ipAddress = NULL;
    char key = 0;

    struct MXCommon__ByteArrayResponse s_ByteArrayResponse;
    struct MSXE351x__AnalogOutputDiagnosticResponse DiagnoseResponse;
    struct MSXE351x__Response Response;

    /* Print sample information */
    printf("+---------------------------------------------------+\n");
    printf("| MSX-E351x                                         |\n");
    printf("| Analog output diagnose sample 	                |\n");
    printf("+---------------------------------------------------+\n");
    printf("\n");

    if (argc != 2)
    {
        printf("\nThe command should be like this : ./WriteOne XXX.XXX.XXX.XXX\n\n");
        exit(1);
    }

    /* manage the signals */
    setup_signal();

    /* allocate memory for the endPoint */
    memset(endPoint, 0, sizeof(endPoint));

    /* this structure the current SOAP connection */
    mx_soap_init(&soapEnv);

    /* get the IP address */
    ipAddress = strdup(argv[1]);

    if (snprintf(endPoint, sizeof(endPoint), "%s:5555", ipAddress) < 0)
    {
        fprintf(stderr, "error: %s:%d!\n", __FUNCTION__, __LINE__);
        exit(1);
    }

    /* ask the module for its type */

    assert_gsoap(&soapEnv, soap_call_MXCommon__GetModuleType(&soapEnv, endPoint, NULL, &DUMMY,&response));
    printf("Returned string: %s\n", response.sArray.__ptr);

    /* who is currently connected ? */
    assert_gsoap(&soapEnv,soap_call_MXCommon__GetClientConnections( &soapEnv,NULL, NULL, &DUMMY,&response));
    assert_soap_command(&soapEnv,&(response.sResponse) );
    printf("Current connections: %s\n\n", response.sArray.__ptr);

    /* Write a value to a channel */
    writeOneFunction(&soapEnv, endPoint, nbrOfChannel);

    /* who is currently connected ? */
    assert_gsoap(&soapEnv,soap_call_MXCommon__GetClientConnections( &soapEnv,NULL, NULL, &DUMMY,&response));
    assert_soap_command(&soapEnv,(struct DefaultResponse*)&(response.sResponse) );
    printf("Current connections: %s\n\n", response.sArray.__ptr);

    assert_gsoap(&soapEnv,soap_call_MSXE351x__AnalogOutputDiagnostic( &soapEnv,endPoint, NULL, 0,&DiagnoseResponse));
    assert_soap_command(&soapEnv,(struct DefaultResponse*)&(DiagnoseResponse.sResponse) );

    if ((DiagnoseResponse.sResponse.iReturnValue == 0))
    {
        if ((DiagnoseResponse.ulOverTemperature != 0) || (DiagnoseResponse.ulShortCircuitOROpenLoad != 0))
        {
            printf("\nDiagnose Error occur :");
            if (DiagnoseResponse.ulOverTemperature != 0)
                printf("\nOver Temperature channel mask in hex: %lX", DiagnoseResponse.ulOverTemperature);
            if (DiagnoseResponse.ulShortCircuitOROpenLoad != 0)
                printf("\nShort-circuit / Open-load channel mask in hex: %lX", DiagnoseResponse.ulShortCircuitOROpenLoad);

            printf("\nPress a key to rearm the analog output diagnostic");
            key = getch();

            assert_gsoap(&soapEnv,soap_call_MSXE351x__AnalogOutputRearmDiagnostic( &soapEnv,endPoint, NULL, 0,&Response));
            assert_soap_command(&soapEnv,(struct DefaultResponse *)&(Response));

            printf("\nRearm OK");
            printf("\n");
            key = 0;
        }
        else
        {
            printf("\rDiagnose information : OK");
        }

    }
    else
    {
        printf("\nsoap_call_MXCommon__GetModuleType SOAP, Response error :%d",
               s_ByteArrayResponse.sResponse.iReturnValue);
        printf("\nSOAP Error : %s", *soap_faultstring(&soapEnv));
    }

    /* Print End programme informations */
    printf("\n\n");
    printf("+---------------------------------------------------+\n");
    printf("| Program ended                                     |\n");
    printf("+---------------------------------------------------+\n");

    return 0;
}
